//
//	srcinfov: compute for each instrument where in the FOV a specified
//			  celestial source will appear
//
//	USAGE:
//		strbs strra=<ra> strdec=<dec> apos=<apos> withsrcoord=true|false
//			  srcra=<ra> srcdec=<dec>
//

#include <iostream>
#include <iomanip>

#include "CalServer.h"
#include "Celestial.h"
#include "errstr.h"
#include "Xmm.h"
#include "param.h"

#include "srcinfov.h"

void srcinfov()
{
	//
	//	construct attitude matrix
	//
	RightAscension ra(stringParameter("strra"));
	Declination dec(stringParameter("strdec"));
	double apos = realParameter("apos")*DEG_TO_RAD;
	SpacecraftAttitude SCA(ra.angle(), dec.angle(), apos);

	if (booleanParameter("withsrccoord")) {
		ra = RightAscension(stringParameter("srcra"));
		dec = Declination(stringParameter("srcdec"));
	}
	CartesianVector srcVec = EquatorialDirection(ra, dec).toUnitCartesian();
    for (unsigned i = 0; i < xmmPayLoad.instruments().size(); ++i) {
        XmmInstrument *ins = xmmPayLoad.instruments()[i];
        if (ins->id() == Xmm::ERM)
            continue;

		calServer.setInstrument(ins->id());
		Boresight *bs = calServer.getAtom(bs);
		bs->withSacCoordOrientation(SCA);

		XmmTelescopePointList pl = bs->projectOntoTelCoord(srcVec);
		XmmCartesianPointList cpl = pl.toCamCoord2().toCamCoord1();
		vector<Ccd> ccd = cpl.onCcd();
		XmmPixelList ccl = cpl.toChipCoord();
		cout<<setw(5)<<Xmm::toString(ins->id())<<": theta = "
			<<setprecision(5)<<pl.theta()[0]<<"'' phi = "
			<<setprecision(5)<<pl.phi()[0]*RAD_TO_DEG
			<<"deg";
		if (ins->id() != Xmm::RGS1 && ins->id() != Xmm::RGS2) {
			cout<<" - this is ";
			if (ccd[0] != Xmm::CCD_NONE)
				cout<<"on pixel "<<ccl.x()[0]<<'/'<<ccl.y()[0]
					<<" of CCD #"<<ccd[0];
			else
				cout<<"not on any CCD (either outside FOV or in chip gap)";
		}
		cout<<endl;
	}
}
